#include "ELF.h"

constexpr ehs::Char_8 Elf_Magic[] = "\x7F" "ELF";
constexpr ehs::UInt_8 Elf_Magic_Size = sizeof(Elf_Magic) - 1;

void WriteHeader(const ElfHeader& header, ehs::Serializer<ehs::UInt_64>& data)
{
	data.Resize(Elf_Magic_Size);
	ehs::Util::Copy(&data[0], Elf_Magic, Elf_Magic_Size);
	data.SetOffset(Elf_Magic_Size);

	data.Write(header.architecture);
	data.Write(header.endianness);
	data.Write(header.elfV1);
	data.Write(header.targetABI);
	data.Write(header.abiV);

	data.Resize(data.Size() + 7);
	ehs::Util::Zero(&data[data.GetOffset()], 7);
	data.SetOffset(data.GetOffset() + 7);

	data.Write(header.elfType);
	data.Write(header.machineArch);
	data.Write(header.elfV2);
	data.Write(header.entryAddress);
	data.Write(header.programsOffset);
	data.Write(header.sectionsOffset);
	data.Write(header.flags);
	data.Write(header.headerSize);
	data.Write(header.pHeaderSize);
	data.Write(header.pEntries);
	data.Write(header.sSize);
	data.Write(header.sEntries);
	data.Write(header.strTabIndex);
}

void WriteHeader(const ElfPrgHeader& header, ehs::Serializer<ehs::UInt_64>& data)
{
	data.Write(header.type);
	data.Write(header.flags);
	data.Write(header.offset);
	data.Write(header.vAddr);
	data.Write(header.pAddr);
	data.Write(header.fileSize);
	data.Write(header.memSize);
	data.Write(header.align);
}

void WriteHeader(const ElfSecHeader& header, ehs::Serializer<ehs::UInt_64>& data)
{
	data.Write(header.index);
	data.Write(header.type);
	data.Write(header.flags);
	data.Write(header.vAddr);
	data.Write(header.offset);
	data.Write(header.size);
	data.Write(header.associatedIndex);
	data.Write(header.info);
	data.Write(header.align);
	data.Write(header.entrySize);
}